local orderedList = {"EXP","Gold","BagSpace","Time","FPS","SkillPoints","Location","Zone","BankMoney","RepairCost","Mail","AP","AchievePoint","VP","HorseTimer","SpecXP","AvA","XPBar"}
local MSWSavedVars = MSWSavedVars or {}
local EM = EVENT_MANAGER
MyStatWin.strLen = 0


--[[ convertPosition: Takes input from MyStatWin.updateLocation and converts it to my format ]]--
local function convertPosition(x,y,head)
	local zDir
	local thisX = x * 100		-- ESO format is 0.xxxxxx, this converts to xx.xxx to allow 
	local thisY = y * 100		-- getLocation to format it to xx.xx
	if (head < 0.3926) and (head >= -0.3926) then zDir = " N"			
	elseif (head >= 0.3926) and (head < 1.1781) then zDir = "NW"
	elseif (head >= 1.1781) and (head < 1.9635) then zDir = " W"
	elseif (head >= 1.9635) and (head < 2.7489) then zDir = "SW"
	elseif (head <= -2.7489) or (head >= 2.7489) then zDir = " S"
	elseif (head >= -2.7489) and (head < -1.9635) then zDir = "SE"
	elseif (head >= -1.9635) and (head < -1.1781) then zDir = " E"
	elseif (head >= -1.1781) and (head < -0.3926) then zDir = "NE" 
	end	
	return thisX, thisY, zDir
end

local function RGBToHex(r,g,b)
	r = r <= 1 and r >= 0 and r or 0
	g = g <= 1 and g >= 0 and g or 0
	b = b <= 1 and b >= 0 and b or 0
	return string.format("%02x%02x%02x", r*255, g*255, b*255)
end

--[[ converrTime: converts ms time to hr:mm:ss time ]]--
local function convertTime(a_time)
		local ms_to_sec = a_time / 1000
		local hours = math.floor(ms_to_sec / 3600)
		local remainder = ms_to_sec - (hours * 3600)
		local minutes = math.floor(remainder / 60)
		local seconds = remainder - (minutes * 60)
		return string.format("%02d:%02d:%02d", hours, minutes, seconds)
end

--[[ MyStatWin.updateAP: Modifies AP points when they change ]]--
local function updateAP(code, points, sound, diff)
	MyStatWin.globals.Modules.AP = MSWSavedVars.Settings.modNameColor.."AP: "..MSWSavedVars.Settings.modNumColor..points
end

--[[ MyStatWin.updateBankMoney: tracks money in bank when deposited/withdrawn ]]--
local function updateBankMoney(code, newTotal, oldTotal)
	MyStatWin.globals.Modules.BankMoney = MSWSavedVars.Settings.modNameColor.."Banked: "..MSWSavedVars.Settings.modNumColor..newTotal
end

--[[ MyStatWin.updateShards: Retrieves the current number of skyshards a character has ]]--
local function updateShards(num)
	if num ~= MyStatWin.globals.myShards then
		MyStatWin.globals.myShards = num
		MyStatWin.globals.Modules.SkillPoints = MSWSavedVars.Settings.modNameColor.."SP: "..MSWSavedVars.Settings.modNumColor..GetAvailableSkillPoints('player')..MSWSavedVars.Settings.modPuncColor.." ("..MSWSavedVars.Settings.modTextColor.."Shards: "..MSWSavedVars.Settings.modPuncColor..num..MSWSavedVars.Settings.modPuncColor..")"
	end
end

--[[ MyStatWin.updateRepairCost: Tracks repair costs for all equipment due to durability loss from death ]]--
local function updateRepairCost()
	MyStatWin.globals.Modules.RepairCost = MSWSavedVars.Settings.modNameColor.."Repair Cost: "..MSWSavedVars.Settings.modNumColor..GetRepairAllCost()
end

--[[ MyStatWin.updateEXP: Updates current character xp and max xp whenever an xp change occurs ]]--
local function updateEXP(tag, unit, xp, myMax, reason)
	if tonumber(GetUnitXPMax('player')) ~= 0 then
		MyStatWin.globals.Modules.EXP = MSWSavedVars.Settings.modNameColor.."XP: "..MSWSavedVars.Settings.modNumColor..xp..MSWSavedVars.Settings.modPuncColor.."/"..MSWSavedVars.Settings.modNumColor..myMax
		MyStatWin.globals.Data.xpPct = math.floor((xp/myMax)*100)
	else
		if tonumber(GetUnitVeteranPointsMax('player')) ~= 0 then
			MyStatWin.globals.Modules.EXP = MSWSavedVars.Settings.modNameColor.."VP: "..MSWSavedVars.Settings.modNumColor..GetUnitVeteranPoints('player')..MSWSavedVars.Settings.modPuncColor.."/"..MSWSavedVars.Settings.modNumColor..GetUnitVeteranPointsMax('player')
		else
			MyStatWin.globals.Modules.EXP = MSWSavedVars.Settings.modNameColor.."EXP: "..MSWSavedVars.Settings.modTextColor.."Max Level"
		end
	end
end

--[[ MyStatWin.updateVP: tracks veteran points ]]--
local function updateVP(tag, unit, current, max, reason)
	if (current ~= 0) then 
		updateEXP(tag,nil,current,max,reason)
	end
end

--[[ MyStatWin.updateFramerate: Retrieves current framerate ]]--
local function updateFramerate()
	MyStatWin.globals.Modules.FPS = MSWSavedVars.Settings.modNameColor.."FPS: "..MSWSavedVars.Settings.modNumColor..math.floor(GetFramerate())
end

--[[ MyStatWin.updateAchievePoint: Tracks current achievement points and compares to total available ]]--
local function updateAchievePoint(name, points, id, link)
	MyStatWin.globals.Modules.AchievePoint = MSWSavedVars.Settings.modNameColor.."Achieve: "..MSWSavedVars.Settings.modNumColor..GetEarnedAchievementPoints()..MSWSavedVars.Settings.modPuncColor.."/"..MSWSavedVars.Settings.modNumColor..GetTotalAchievementPoints()
end

--[[ MyStatWin.updateTime: Gets current local time in nice format ]]--
local function updateTime()
	MyStatWin.globals.Modules.Time = MSWSavedVars.Settings.modNameColor.."Time: "..MSWSavedVars.Settings.modNumColor..GetTimeString()
end

--[[ MyStatWin.getLocation updates current player location each frame update. Position is converted
	from game's 0.xxxxx format to xx.xx for x and y coordinates and current heading
	converted from radians to a string direction (currently N/S/E/W displays as TN/TS/TW/TW
	to keep string as even as possible) ]]--
local function getLocation()
	local xPos,yPos,zPos = GetMapPlayerPosition('player')
	local xLoc,yLoc,dir = convertPosition(xPos,yPos,zPos)
	MyStatWin.globals.Modules.Location = MSWSavedVars.Settings.modNameColor.."Loc: "..MSWSavedVars.Settings.modNumColor..(string.format("%4.2f,%4.2f,%2s",xLoc,yLoc,dir))
end

--[[ MyStatWin.BagUpdate: Updates number of slots in your character's bag whenever bag contents are changed ]]--
local function BagUpdate()
	local bagSlots = GetBagSize(1)
	local freeBagSpace = GetNumBagUsedSlots(1)
	MyStatWin.globals.Modules.BagSpace = MSWSavedVars.Settings.modNameColor.."Inv: "..MSWSavedVars.Settings.modNumColor..freeBagSpace..MSWSavedVars.Settings.modPuncColor.."/"..MSWSavedVars.Settings.modNumColor..bagSlots
end

--[[ MyStatWin.MoneyUpdate: Updates character gold in inventory when it changes ]]--
local function MoneyUpdate(code, nMon, oMon, whyMon)
	MyStatWin.globals.Modules.Gold = MSWSavedVars.Settings.modNameColor.."Gold: "..MSWSavedVars.Settings.modNumColor..GetCurrentMoney()
end

--[[ MyStatWin.SkillPointUpdate: Updates skill points available whenever that number changes ]]--
local function SkillPointUpdate(pre, post, fromShard)
		 MyStatWin.globals.Modules.SkillPoints = MSWSavedVars.Settings.modNameColor.."SP: "..MSWSavedVars.Settings.modNumColor..GetAvailableSkillPoints('player')..MSWSavedVars.Settings.modPuncColor.." ("..MSWSavedVars.Settings.modTextColor.."Shards: "..MSWSavedVars.Settings.modNumColor..GetNumSkyShards()..MSWSavedVars.Settings.modPuncColor..")"
end

--[[ MyStatWin.UpdateZone: Changes current subzone as you move in to or out of them. The format
	is Zone (Subzone), for example Stonefalls (Kragenmoor) would indicate you are in the wide zone
	of Stonefalls, with Kragenmoor the area of Stonefalls you are in ]]--
local function UpdateZone(test,zone, sub, isItNew)
	local zoneName = GetUnitZone('player')
	if (sub ~= "") and (sub ~= zoneName) then
		MyStatWin.globals.Modules.Zone = MSWSavedVars.Settings.modNameColor.."Zone: "..MSWSavedVars.Settings.modTextColor..zoneName..MSWSavedVars.Settings.modPuncColor.." ("..MSWSavedVars.Settings.modTextColor..sub..MSWSavedVars.Settings.modPuncColor..")"
	else
		MyStatWin.globals.Modules.Zone = MSWSavedVars.Settings.modNameColor.."Zone: "..MSWSavedVars.Settings.modTextColor..zoneName
	end	
end

local function UpdateZoneLoad()
	UpdateZone(nil,GetUnitZone('player'),GetPlayerLocationName(),nil)
end

--[[ MyStatWin.updateMail: Informs you if you have unread mail items ]]--
local function updateMail(code,unread)
	if (unread == 0) then MyStatWin.globals.Modules.Mail = MSWSavedVars.Settings.modNameColor.."Mail: "..MSWSavedVars.Settings.modTextColor.."No" else MyStatWin.globals.Modules.Mail = MSWSavedVars.Settings.modNameColor.."Mail: "..MSWSavedVars.Settings.modTextColor.."Yes" end
end

--[[ MyStatWin.updateHorseTimer: If player has an active mount, monitors time until mount must be fed,
	else informs of no mount ]]--
local function updateHorseTimer()
	local timer = GetTimeUntilMountCanBeFed()
	if (not IsStableSlotEmpty(1)) then
		if timer ~= nil then
			local formattedTime = convertTime(timer)
			MyStatWin.globals.Modules.HorseTimer = MSWSavedVars.Settings.modNameColor.."Feed Horse: "..MSWSavedVars.Settings.modNumColor..formattedTime
		else
			MyStatWin.globals.Modules.HorseTimer = MSWSavedVars.Settings.modTextColor.."Level 50 Mount"
		end
	else
		MyStatWin.globals.Modules.HorseTimer = MSWSavedVars.Settings.modTextColor.."No Active Mount"
	end
end

--[[ MyStatWin.updateSpecXP: Monitors and updates vampire/werewolf xp ]]--
local function updateSpecXP()
	if (GetNumSkillLines( SKILL_TYPE_WORLD ) == 0) or (GetNumSkillLines(SKILL_TYPE_WORLD)==1) then
		MyStatWin.globals.Modules.SpecXP = MSWSavedVars.Settings.modTextColor.."No Vamp/Wolf XP"
	else
		local name, rank = GetSkillLineInfo( SKILL_TYPE_WORLD, 2 )
		local oldxp, xplvl, xp = GetSkillLineXPInfo( SKILL_TYPE_WORLD, 2 )
		if xp == nil or xplvl == nil or oldxp == nil then 
			MyStatWin.globals.Modules.SpecXP = MSWSavedVars.Settings.modTextColor.."No Vamp/Wolf XP"
		else
			xp = xp - oldxp
			xplvl = xplvl - oldxp
			local percentage = string.format("%02d",xp/xplvl)
			MyStatWin.globals.Modules.SpecXP = MSWSavedVars.Settings.modNameColor..name.." XP: "..MSWSavedVars.Settings.modNumColor..percentage..MSWSavedVars.Settings.modPuncColor.."%"
		end
	end
end

local function updateAvA(code,tag,points,diff)
	local rank,sub = GetUnitAvARank('player')
	local rankName = GetAvARankName(GetUnitGender('player'),rank)
	local curPoints = GetUnitAvARankPoints('player')
	local subStart, subEnd, rankStart, rankEnd = GetAvARankProgress(curPoints)
	MyStatWin.globals.Modules.AvA = MSWSavedVars.Settings.modNameColor.."AvA: "..MSWSavedVars.Settings.modNumColor..curPoints..MSWSavedVars.Settings.modPuncColor.." ("..MSWSavedVars.Settings.modTextColor..rankName..MSWSavedVars.Settings.modPuncColor..")/"..MSWSavedVars.Settings.modNumColor..rankEnd
end

local function checkForHide(count)
	if count == 0 then 
		MSWSavedVars.Settings.barDimensions.X = 25
		MyStatusWindow:SetHidden(true)
		MyStatWin_BG2:SetHidden(true)
		MyStatWin_BG:SetHidden(true)
	else
		MyStatusWindow:SetHidden(false)
		MyStatWin_BG2:SetHidden(false)
		MyStatWin_BG:SetHidden(false)
	end
end

local function initCounter()
	local accountName = GetDisplayName()
	local currentCount = 0
	for _,k in ipairs(orderedList) do
		if (MSWSavedVars.Modules[k]) then
			currentCount = currentCount + 1
		end
	end
	MyStatWin.globals.Data.counter = currentCount
	if currentCount > 1 then MSWSavedVars.Settings.moduleCount = currentCount - 1 else MSWSavedVars.Settings.moduleCount = 0 end
end

function MyStatWin.initializeGlobals()
	local unitID = "player"
	MSWSavedVars.Settings.modNameColor = "|c"..RGBToHex(MSWSavedVars.Settings.modNameColorRGB.r,MSWSavedVars.Settings.modNameColorRGB.g,MSWSavedVars.Settings.modNameColorRGB.b)
	MyStatWin.currentPlayer = GetUnitName(unitID)
	BagUpdate()	
	UpdateZone(nil,nil,GetPlayerLocationName(),nil)
	MoneyUpdate(nil,nil,nil,nil)	
	updateEXP(nil,nil,GetUnitXP(unitID),GetUnitXPMax(unitID),nil)
	MyStatWin.globals.Data.xpPct = math.floor((GetUnitXP(unitID)/GetUnitXPMax(unitID))*100)
	updateAP(nil,GetAlliancePoints(),nil,nil)
	updateBankMoney(nil,GetBankedMoney(),nil)
	updateAchievePoint(nil,nil,nil,nil)
	updateMail(nil,GetNumUnreadMail())
	updateAvA(nil,nil,GetUnitAvARankPoints('player'),nil)
	SkillPointUpdate(nil,nil,nil)
	initCounter()
	MyStatWin.buildToolbar()
end

function MyStatWin.registerEvents()
	MyStatWin.initializeGlobals()
	EM:RegisterForEvent("MyStatWin", EVENT_MONEY_UPDATE, MoneyUpdate)				
	EM:RegisterForEvent("MyStatWin", EVENT_INVENTORY_SINGLE_SLOT_UPDATE, BagUpdate)	
	EM:RegisterForEvent("MyStatWin", EVENT_SKILL_POINTS_CHANGED, SkillPointUpdate)	
	EM:RegisterForEvent("MyStatWin", EVENT_ZONE_CHANGED, UpdateZone)				
	EM:RegisterForEvent("MyStatWin", EVENT_MAIL_NUM_UNREAD_CHANGED, updateMail)		
	EM:RegisterForEvent("MyStatWin", EVENT_EXPERIENCE_UPDATE, updateEXP)			
	EM:RegisterForEvent("MyStatWin", EVENT_ALLIANCE_POINT_UPDATE, updateAP)			
	EM:RegisterForEvent("MyStatWin", EVENT_BANKED_MONEY_UPDATE, updateBankMoney)	
	EM:RegisterForEvent("MyStatWin", EVENT_VETERAN_POINTS_UPDATE, updateVP)
	EM:RegisterForEvent("MyStatWin", EVENT_ACHIEVEMENT_AWARDED, updateAchievePoint)
	EM:RegisterForEvent("MyStatWin", EVENT_PLAYER_DEAD, updateRepairCost)
	EM:RegisterForEvent("MyStatWin", EVENT_RANK_POINT_UPDATE, updateAvA)
	EM:RegisterForEvent("MyStatWin", EVENT_LINKED_WORLD_POSITION_CHANGED, UpdateZoneLoad)
end

function MyStatWin.buildToolbar()
	toolText:SetText("")
	MyStatWin.globals.Data.barString = ""
	local spacer = " - "
	local currentCount = MyStatWin.globals.Data.counter
	checkForHide(currentCount)
	for _,k in ipairs(orderedList) do
		if (MSWSavedVars.Modules[k]) then
			currentCount = currentCount - 1
			if (k == "Location") then getLocation() end
			if (k == "SkillPoints") then updateShards(GetNumSkyShards()) end
			if (k == "FPS") then updateFramerate() end
			if (k == "Time") then updateTime() end
			if (k == "RepairCost") then updateRepairCost() end
			if (k == "HorseTimer") then updateHorseTimer() end
			if (k == "SpecXP") then updateSpecXP() end
			if (k == "XPBar") then MSW_XPBar_BG2:SetDimensions(MyStatWin.globals.Data.xpPct,10) end
			if (currentCount ~= 0) then
				if (k == "XPBar") then
					MyStatWin.globals.Data.barString = MyStatWin.globals.Data.barString .. spacer
				else
					MyStatWin.globals.Data.barString = MyStatWin.globals.Data.barString..MyStatWin.globals.Modules[k]..MSWSavedVars.Settings.modSpaceColor..spacer
				end
			else
				if (k ~= "XPBar") then
					MyStatWin.globals.Data.barString = MyStatWin.globals.Data.barString..MyStatWin.globals.Modules[k]
				end
			end
		end
	end
	toolText:SetText(MyStatWin.globals.Data.barString)
	local x,y = toolText:GetDimensions()
	MSWSavedVars.Settings.barDimensions.X = x + 20
	MyStatusWindow:SetDimensions(MSWSavedVars.Settings.barDimensions.X,MSWSavedVars.Settings.barDimensions.Y)
	MyStatWin_BG2:SetDimensions(MSWSavedVars.Settings.barDimensions.X,6)
	MyStatWin_BG:SetDimensions(MSWSavedVars.Settings.barDimensions.X,20)
end

local function Init(code, name)
	if name == "MyStatusWindow" then
		EM:UnregisterForEvent("MSW_Modules",code)
		MSWSavedVars = MyStatWin.setupSavedVars()
	end
end

EM:RegisterForEvent( "MSW_Modules" , EVENT_ADD_ON_LOADED , Init )