local LAM2 = LibStub:GetLibrary("LibAddonMenu-2.0")
local LMP = LibStub:GetLibrary("LibMediaProvider-1.0")
local newFont = LMP:Register('font',"DejaVu Sans Mono", "MyStatusWindow/DejaVuSansMono.ttf")
local fontList = LMP:List('font')
local fontStyles = {"normal", "outline", "thick-outline", "shadow", "soft-shadow-thick", "soft-shadow-thin"}
local MSWSavedVars = MSWSavedVars or {}
local EM = EVENT_MANAGER
local orderedList = {"EXP","Gold","BagSpace","Time","FPS","SkillPoints","Location","Zone","BankMoney","RepairCost","Mail","AP","AchievePoint","HorseTimer","SpecXP","AvA","XPBar"}

local function reloadBar()
	toolText:SetFont(("%s|%s|%s"):format(LMP:Fetch('font', MSWSavedVars.Settings.usedFont), MSWSavedVars.Settings.currFontSize, MSWSavedVars.Settings.currStyle))
	if not MSWSavedVars.Settings.dragBarLoc then
		MyStatusWindow:ClearAnchors()
		MyStatusWindow:SetAnchor(TOPLEFT,GuiXml,TOPLEFT,MSWSavedVars.Settings.barOffset.X,MSWSavedVars.Settings.barOffset.Y)
		MyStatWin_BG2:ClearAnchors()
		MyStatWin_BG2:SetAnchor(TOPLEFT,MyStatusWindow,TOPLEFT)
		MyStatWin_BG:ClearAnchors()
		MyStatWin_BG:SetAnchor(TOPLEFT,MyStatWin_BG2,BOTTOMLEFT)
	else
		MyStatusWindow:ClearAnchors()
		MyStatusWindow:SetAnchor(TOPLEFT,GuiXml,TOPLEFT,MSWSavedVars.Settings.barOffset.X,MSWSavedVars.Settings.barOffset.Y)
		MyStatusWindow:SetDimensions(MSWSavedVars.Settings.barDimensions.X,MSWSavedVars.Settings.barDimensions.Y)
		MyStatWin_BG:ClearAnchors()
		MyStatWin_BG:SetAnchor(TOPLEFT,MyStatusWindow,TOPLEFT)
		MyStatWin_BG2:ClearAnchors()
		MyStatWin_BG2:SetAnchor(TOPLEFT,MyStatWin_BG,BOTTOMLEFT)
	end
end

local function RGBToHex(r,g,b)
	r = r <= 1 and r >= 0 and r or 0
	g = g <= 1 and g >= 0 and g or 0
	b = b <= 1 and b >= 0 and b or 0
	return string.format("%02x%02x%02x", r*255, g*255, b*255)
end

local panelData = {
	type = "panel",
	name = "MyStatusWindow",
	author = "Skyraker",
	version = "3.2",
	slashCommand = "/ms"
}

local optionsData = {
	[1] = {
		type = "header",
		name = "General Options"
	},
	[2] = {
		type = "checkbox",
		name = "Lock position",
		tooltip = "Locks info bar in position.",
		getFunc = function() return MSWSavedVars.Settings.locked end,
		setFunc = function(value)
								MSWSavedVars.Settings.locked = value
								MyStatWin_BG2:SetMovable(not MSWSavedVars.Settings.locked)
							end
	},
	[3] = {
		type = "checkbox",
		name = "Enable/Disable Compass Bump",
		tooltip = "Allow/prevent auto compass bumping.",
		getFunc = function() return MSWSavedVars.Settings.bumpLock end,
		setFunc = function(value)
								MSWSavedVars.Settings.bumpLock = value
							end
	},
	[4] = {
		type = "checkbox",
		name = "Drag Bar on Bottom",
		tooltip = "Place Drag Bar below Info Bar.",
		getFunc = function() return MSWSavedVars.Settings.dragBarLoc end,
		setFunc = function(value)
								MSWSavedVars.Settings.dragBarLoc = value
								reloadBar()
							end
	},
	[5] = {
		type = "header",
		name = "Bar Options"
	},
	[6] = {
		type = "submenu",
		name = "Drag Bar Options",
		tooltip = "Modify Drag Bar Options",
		controls = {
			[1] = {
				type = "slider",
				name = "Transparency",
				tooltip = "Change transparency of the drag bar.",
				min = 0,
				max = 10,
				step = 1,
				getFunc = function() return MyStatWin_BG2:GetAlpha() * 10 end,
				setFunc = function(value)
										MSWSavedVars.Settings.barColor.alpha = value / 10.0
										MyStatWin_BG2:SetAlpha(value/10.0)
									end,
				width = "half"
			},
			[2] = {
				type = "colorpicker",
				name = "Drag Bar Color",
				tooltip = "Modify color of drag bar.",
				getFunc = function() return MSWSavedVars.Settings.barColor.r,MSWSavedVars.Settings.barColor.g,MSWSavedVars.Settings.barColor.b,MSWSavedVars.Settings.barColor.a end,
				setFunc = function(r,g,b,a)
										MSWSavedVars.Settings.barColor.r = r
									MSWSavedVars.Settings.barColor.g = g
									MSWSavedVars.Settings.barColor.b = b
									MSWSavedVars.Settings.barColor.a = a
									MyStatWin_BG2:SetCenterColor(r,g,b,a)
								end,
				width = "half"
			}
		}
	},
	[7] = {
		type = "submenu",
		name = "Selectable Modules",
		tooltip = "Select which info to display.",
		controls = {
			[1] = {
				type = "checkbox",
				name = "Location",
				tooltip = "Show current player's position.",
				getFunc = function() return MSWSavedVars.Modules.Location end,
				setFunc = function(value)
										if value then
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter + 1
										else
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter - 1
										end
										MSWSavedVars.Modules.Location = value
									end,
				width = "half"
			},
			[2] = {
				type = "checkbox",
				name = "Gold",
				tooltip = "Show gold on hand.",
				getFunc = function() return MSWSavedVars.Modules.Gold end,
				setFunc = function(value)
										if value then
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter + 1
										else
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter - 1
										end
										MSWSavedVars.Modules.Gold = value
									end,
				width = "half"
			},
			[3] = {
				type = "checkbox",
				name = "Inventory Space",
				tooltip = "Show Used/Total Inventory Space",
				getFunc = function() return MSWSavedVars.Modules.BagSpace end,
				setFunc = function(value)
										if value then
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter + 1
										else
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter - 1
										end
										MSWSavedVars.Modules.BagSpace = value
									end,
				width = "half"
			},
			[4] = {
				type = "checkbox",
				name = "Skill Points",
				tooltip = "Show unused Skill Points and Skyshard tally.",
				getFunc = function() return MSWSavedVars.Modules.SkillPoints end,
				setFunc = function(value)
										if value then
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter + 1
										else
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter - 1
										end
										MSWSavedVars.Modules.SkillPoints = value
									end,
				width = "half"
			},
			[5] = {
				type = "checkbox",
				name = "Experience",
				tooltip = "Show Experience/Next Level Experience Required.",
				getFunc = function() return MSWSavedVars.Modules.EXP end,
				setFunc = function(value)
										if value then
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter + 1
										else
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter - 1
										end
										MSWSavedVars.Modules.EXP = value
									end,
				width = "half"
			},
			[6] = {
				type = "checkbox",
				name = "Repair Cost",
				tooltip = "Show Cost To Repair Equipment.",
				getFunc = function() return MSWSavedVars.Modules.RepairCost end,
				setFunc = function(value)
										if value then
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter + 1
										else
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter - 1
										end
										MSWSavedVars.Modules.RepairCost = value
									end,
				width = "half"
			},
			[7] = {
				type = "checkbox",
				name = "Banked Money",
				tooltip = "Show Money deposited in Bank.",
				getFunc = function() return MSWSavedVars.Modules.BankMoney end,
				setFunc = function(value)
										if value then
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter + 1
										else
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter - 1
										end
										MSWSavedVars.Modules.BankMoney = value
									end,
				width = "half"
			},
			[8] = {
				type = "checkbox",
				name = "Local Time",
				tooltip = "Show Current Local Time.",
				getFunc = function() return MSWSavedVars.Modules.Time end,
				setFunc = function(value)
										if value then
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter + 1
										else
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter - 1
										end
										MSWSavedVars.Modules.Time = value
									end,
				width = "half"
			},
			[9] = {
				type = "checkbox",
				name = "Framerate",
				tooltip = "Show FPS.",
				getFunc = function() return MSWSavedVars.Modules.FPS end,
				setFunc = function(value)
										if value then
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter + 1
										else
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter - 1
										end
										MSWSavedVars.Modules.FPS = value
									end,
				width = "half"
			},
			[10] = {
				type = "checkbox",
				name = "Wolf/Vamp XP",
				tooltip = "Show Non-Human XP percentage.",
				getFunc = function() return MSWSavedVars.Modules.SpecXP end,
				setFunc = function(value)
										if value then
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter + 1
										else
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter - 1
										end
										MSWSavedVars.Modules.SpecXP = value
									end,
				width = "half"
			},
			[11] = {
				type = "checkbox",
				name = "Horse Timer",
				tooltip = "Display time until active mount can be fed.",
				getFunc = function() return MSWSavedVars.Modules.HorseTimer end,
				setFunc = function(value)
										if value then
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter + 1
										else
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter - 1
										end
										MSWSavedVars.Modules.HorseTimer = value
									end,
				width = "half"
			},
			[12] = {
				type = "checkbox",
				name = "Zone",
				tooltip = "Show Player's Current Zone.",
				getFunc = function() return MSWSavedVars.Modules.Zone end,
				setFunc = function(value)
										if value then
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter + 1
										else
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter - 1
										end
										MSWSavedVars.Modules.Zone = value
									end,
				width = "half"
			},
			[13] = {
				type = "checkbox",
				name = "Unread Mail",
				tooltip = "Notify Player of unread mail.",
				getFunc = function() return MSWSavedVars.Modules.Mail end,
				setFunc = function(value)
										if value then
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter + 1
										else
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter - 1
										end
										MSWSavedVars.Modules.Mail = value
									end,
				width = "half"
			},
			[14] = {
				type = "checkbox",
				name = "Alliance Points",
				tooltip = "Show Player's Alliance Point Total.",
				getFunc = function() return MSWSavedVars.Modules.AP end,
				setFunc = function(value)
										if value then
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter + 1
										else
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter - 1
										end
										MSWSavedVars.Modules.AP = value
									end,
				width = "half"
			},
			[15] = {
				type = "checkbox",
				name = "Achievement Points",
				tooltip = "Show Current Achievement Points Earned/Max Points Available.",
				getFunc = function() return MSWSavedVars.Modules.AchievePoint end,
				setFunc = function(value)
										if value then
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter + 1
										else
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter - 1
										end
										MSWSavedVars.Modules.AchievePoint = value
									end,
				width = "half"
			},
			[16] = {
				type = "checkbox",
				name = "AvA Points",
				tooltip = "Show Player's AvA points/rank.",
				getFunc = function() return MSWSavedVars.Modules.AvA end,
				setFunc = function(value)
										if value then
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter + 1
										else
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter - 1
										end
										MSWSavedVars.Modules.AvA = value
									end,
				width = "half"
			},
			[17] = {
				type = "checkbox",
				name = "XP Bar",
				tooltip = "Display Graphical XP Bar.",
				getFunc = function() return MSWSavedVars.Modules.XPBar end,
				setFunc = function(value)
										MSW_XPBar:SetHidden(not value)
										if value then
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter + 1
										else
											MyStatWin.globals.Data.counter = MyStatWin.globals.Data.counter - 1
										end
										MSWSavedVars.Modules.XPBar = value
									end,
				width = "half"
			}
		}
	},
	[8] = {
		type = "submenu",
		name = "Text/Background Options",
		tooltip = "Change font settings and background transparency.",
		controls = {
			[1] = {
				type = "slider",
				name = "Background Transparency",
				tooltip = "Adjust transparency of info bar background.",
				min = 0,
				max = 10,
				step = 1,
				getFunc = function() return MSWSavedVars.Settings.currAlpha * 10 end,
				setFunc = function(value)
										MSWSavedVars.Settings.currAlpha = value/10.0
										MyStatWin_BG:SetAlpha(value/10.0)
									end,
				width = "half"
			},
			[2] = {
				type = "dropdown",
				name = "Font",
				tooltip = "Change displayed font.",
				choices = fontList,
				getFunc = function() return MSWSavedVars.Settings.usedFont end,
				setFunc = function(value)
										MSWSavedVars.Settings.usedFont = value
										reloadBar()
									end,
				width = "half"
			},
			[3] = {
				type = "slider",
				name = "Size",
				tooltip = "Change Font Size.",
				min = 10,
				max = 20,
				step = 2,
				getFunc = function() return MSWSavedVars.Settings.currFontSize end,
				setFunc = function(value)
										local numSpacers = 0
										local currentCount = 0
										MSWSavedVars.Settings.barDimensions.X = 25
										MSWSavedVars.Settings.currFontSize = value
										strFontSize = tostring(value)
										MSWSavedVars.Settings.fontSizing = MyStatWin.globals.fontSizing[strFontSize]
										for _,k in ipairs(orderedList) do
											if (MSWSavedVars.Modules[k]) then
												currentCount = currentCount + 1
												local modifier = MSWSavedVars.Settings.fontSizing * MyStatWin.globals.moduleSizes[k]
												MSWSavedVars.Settings.barDimensions.X = MSWSavedVars.Settings.barDimensions.X + modifier
											end
										end
										numSpacers = currentCount - 1
										if (numSpacers > 0) then MSWSavedVars.Settings.barDimensions.X = MSWSavedVars.Settings.barDimensions.X + (MyStatWin.globals.Data.spacerSize * numSpacers) end
										reloadBar()
									end,
				width = "half"
			},
			[4] = {
				type = "dropdown",
				name = "Style",
				tooltip = "Change Font Style.",
				choices = fontStyles,
				getFunc = function() return MSWSavedVars.Settings.currStyle end,
				setFunc = function(value)
										MSWSavedVars.Settings.currStyle = value
										reloadBar()
									end,
				width = "half"
			}
		}
	},
	[9] = {
		type = "submenu",
		name = "Colors",
		tooltip = "Modify info bar colors.",
		controls = {
			[1] = {
				type = "colorpicker",
				name = "Module Name",
				tooltip = "Select color for module names.",
				getFunc = function() return MSWSavedVars.Settings.modNameColorRGB.r,MSWSavedVars.Settings.modNameColorRGB.g,MSWSavedVars.Settings.modNameColorRGB.b,MSWSavedVars.Settings.modNameColorRGB.a end,
				setFunc = function(r,g,b,a)
											MSWSavedVars.Settings.modNameColorRGB.r = r
											MSWSavedVars.Settings.modNameColorRGB.g = g
											MSWSavedVars.Settings.modNameColorRGB.b = b
											MSWSavedVars.Settings.modNameColorRGB.a = a
											MSWSavedVars.Settings.modNameColor = "|c"..RGBToHex(r,g,b)
											MyStatWin.initializeGlobals()
										end,
				width = "half"
			},
			[2] = {
				type = "colorpicker",
				name = "Module Numbers",
				tooltip = "Select color for numbers.",
				getFunc = function() return MSWSavedVars.Settings.modNumColorRGB.r,MSWSavedVars.Settings.modNumColorRGB.g,MSWSavedVars.Settings.modNumColorRGB.b,MSWSavedVars.Settings.modNumColorRGB.a end,
				setFunc = function(r,g,b,a)
											MSWSavedVars.Settings.modNumColorRGB.r = r
											MSWSavedVars.Settings.modNumColorRGB.g = g
											MSWSavedVars.Settings.modNumColorRGB.b = b
											MSWSavedVars.Settings.modNumColorRGB.a = a
											MSWSavedVars.Settings.modNumeColor = "|c"..RGBToHex(r,g,b)
											MyStatWin.initializeGlobals()
										end,
				width = "half"
			},
			[3] = {
				type = "colorpicker",
				name = "Module Text",
				tooltip = "Select color for non-module name text.",
				getFunc = function() return MSWSavedVars.Settings.modTextColorRGB.r,MSWSavedVars.Settings.modTextColorRGB.g,MSWSavedVars.Settings.modTextColorRGB.b,MSWSavedVars.Settings.modTextColorRGB.a end,
				setFunc = function(r,g,b,a)
											MSWSavedVars.Settings.modTextColorRGB.r = r
											MSWSavedVars.Settings.modTextColorRGB.g = g
											MSWSavedVars.Settings.modTextColorRGB.b = b
											MSWSavedVars.Settings.modTextColorRGB.a = a
											MSWSavedVars.Settings.modTextColor = "|c"..RGBToHex(r,g,b)
											MyStatWin.initializeGlobals()
										end,
				width = "half"
			},
			[4] = {
				type = "colorpicker",
				name = "Puncutation",
				tooltip = "Select color for puncuation.",
				getFunc = function() return MSWSavedVars.Settings.modPuncColorRGB.r,MSWSavedVars.Settings.modPuncColorRGB.g,MSWSavedVars.Settings.modPuncColorRGB.b,MSWSavedVars.Settings.modPuncColorRGB.a end,
				setFunc = function(r,g,b,a)
											MSWSavedVars.Settings.modPuncColorRGB.r = r
											MSWSavedVars.Settings.modPuncColorRGB.g = g
											MSWSavedVars.Settings.modPuncColorRGB.b = b
											MSWSavedVars.Settings.modPuncColorRGB.a = a
											MSWSavedVars.Settings.modPuncColor = "|c"..RGBToHex(r,g,b)
											MyStatWin.initializeGlobals()
										end,
				width = "half"
			},
			[5] = {
				type = "colorpicker",
				name = "Spacer",
				tooltip = "Select color for module spacer.",
				getFunc = function() return MSWSavedVars.Settings.modSpaceColorRGB.r,MSWSavedVars.Settings.modSpaceColorRGB.g,MSWSavedVars.Settings.modSpaceColorRGB.b,MSWSavedVars.Settings.modSpaceColorRGB.a end,
				setFunc = function(r,g,b,a)
											MSWSavedVars.Settings.modSpaceColorRGB.r = r
											MSWSavedVars.Settings.modSpaceColorRGB.g = g
											MSWSavedVars.Settings.modSpaceColorRGB.b = b
											MSWSavedVars.Settings.modSpaceColorRGB.a = a
											MSWSavedVars.Settings.modSpaceColor = "|c"..RGBToHex(r,g,b)
											MyStatWin.initializeGlobals()
										end,
				width = "half"
			}
		}
	}
}

LAM2:RegisterAddonPanel("MSWOptions", panelData)
LAM2:RegisterOptionControls("MSWOptions", optionsData)

local function Init(code, name)
	if name == "MyStatusWindow" then
		EM:UnregisterForEvent("MSW_Settings",code)
		MSWSavedVars = MyStatWin.setupSavedVars()
	end
end

EM:RegisterForEvent( "MSW_Settings" , EVENT_ADD_ON_LOADED , Init )